/* cc -O2 -o ice ice.c -lm
 */
#include "../gen/include.h"
#include "../gen/rndgeni.c"
typedef float real;
#include "../gen/vector3d.h"

#define MAXn 8

struct cell_s {
  int n;
  double OO;      /* O-O distance (multiplication factor for a and r[]) */
  vector a;       /* basic cell for O-O distance = 1 */
  vector r[MAXn]; /* molecular positions in units of the basic cell*/ }
Ih = {
  8, 2.767,
  {1.63299316185545,2.82842712474619,8./3},
  {
    {0,  1./6, 3./16},
    {0,  5./6, 5./16},
    {0.5,2./6, 5./16},
    {0.5,4./6, 3./16},
    {0,  1./6,13./16},
    {0,  5./6,11./16},
    {0.5,2./6,11./16},
    {0.5,4./6,13./16}
  }
},
Ic = {
  8, 2.767,
  {2.3094010767585,2.3094010767585,2.3094010767585},
  {
    {0,   0   ,0   },
    {0.5 ,0.5 ,0   },
    {0.25,0.25,0.25},
    {0.75,0.75,0.25},
    {0   ,0.5 ,0.5 },
    {0.5 ,0   ,0.5},
    {0.75,0.25,0.75},
    {0.25,0.75,0.75}
  }
};

vector L;
int N;

inline double sqr(double x)
{
  return x*x;
}

double ddist(float *a,float *b)
{
  int i;
  double rr=0;

  loop (i,0,3)
    rr+=sqr(L[i]/2-fabs(L[i]/2-fabs(a[i]-b[i])));

  return rr;
}

struct cfg_s {
  vector r;
  int nnbr; /* should be 4 */
  int nbr[4]; /* HHee */
} *cfg;

int ndef(int i)
/* number of Bjerrum defects around molecule i */
{
  int n,d=0;

  n=cfg[i].nbr[0]; if (cfg[n].nbr[0]==i || cfg[n].nbr[1]==i) d++;
  n=cfg[i].nbr[1]; if (cfg[n].nbr[0]==i || cfg[n].nbr[1]==i) d++;
  n=cfg[i].nbr[2]; if (cfg[n].nbr[2]==i || cfg[n].nbr[3]==i) d++;
  n=cfg[i].nbr[3]; if (cfg[n].nbr[2]==i || cfg[n].nbr[3]==i) d++;

  return d;
}

int ndef_lt(int i)
/* number of Bjerrum defects around molecule i from molecules <i*/
{
  int n,d=0;

  n=cfg[i].nbr[0]; if (n<i) if (cfg[n].nbr[0]==i || cfg[n].nbr[1]==i) d++;
  n=cfg[i].nbr[1]; if (n<i) if (cfg[n].nbr[0]==i || cfg[n].nbr[1]==i) d++;
  n=cfg[i].nbr[2]; if (n<i) if (cfg[n].nbr[2]==i || cfg[n].nbr[3]==i) d++;
  n=cfg[i].nbr[3]; if (n<i) if (cfg[n].nbr[2]==i || cfg[n].nbr[3]==i) d++;

  return d;
}

vector M; /* dipole moment */
int pass;

double dipmq(double OO)
{
  int i,n,j,k;
  vector dr;
  double mu=2.3052; /* [D] dip.mom. of TIP4P/2005 water */
  double Mcomp=OO*sqrt(4./3); /* dip.mom. of this tetrahedral model */

  VO(M,=0)

  loop (i,0,N) loop (j,0,2) {
    n=cfg[i].nbr[j];
    VVV(dr,=cfg[n].r,-cfg[i].r)
    loop (k,0,3) {
      if (dr[k]<-L[k]/2) dr[k]+=L[k];
      if (dr[k]> L[k]/2) dr[k]-=L[k]; }
    VV(M,+=dr) }

  VO(M,/=Mcomp)
  printf("  M = %.6f %.6f %.6f = %g mu_H2O (pass %d)\n",VARG(M),sqrt(SQR(M)),pass);
  VO(M,*=mu)
  printf("  M = %.6f %.6f %.6f = %g D (for mu_H2O=2.3052 D)\n",VARG(M),mu=sqrt(SQR(M)));

  pass++;
  return mu;
}

int indx(char *arg,int df)
/* "8y" returns 1, "9" returns df */
{
  char c=strend(arg)[-1];
  if (strchr("xyz",c)) return c-'x';
  else return df;
}

struct cell_s *ice;
double Mmid=5e9,Merr=5e9;
int quiet;
char *args;
char *SITES;

void nextcfg(int i,int nr)
{
  int k;

  if (nr&1) {
    k=cfg[i].nbr[0],cfg[i].nbr[0]=cfg[i].nbr[2],cfg[i].nbr[2]=k;
    k=cfg[i].nbr[1],cfg[i].nbr[1]=cfg[i].nbr[3],cfg[i].nbr[3]=k; }
  else {
    k=cfg[i].nbr[0],cfg[i].nbr[0]=cfg[i].nbr[1],cfg[i].nbr[1]=cfg[i].nbr[2],cfg[i].nbr[2]=k; }
}

void annealing(void)
{
  /* simulated annealing if initial random lattice */
  int i,j,k;
  int a,b;
  int old[4];
  double T=1;
  int Uold,Unew;
//    double Mold,Mnew;

  do {

    /* randomize */
    loop (i,0,N)
      loop (j,0,N*100) {
        a=irnd(4),b=irnd(4);
        k=cfg[i].nbr[a], cfg[i].nbr[a]=cfg[i].nbr[b], cfg[i].nbr[b]=k; }

    printf("original dipole moment of the box:\n");
    dipmq(ice->OO);

    /* Monte Carlo assignment of protons by simulated annealing */
    for (;;) {
      int nd=0;
      double exptab0[9],*exptab=exptab0+4;

      loop (i,0,N) nd+=ndef(i);
      nd/=2; /* every defect was included twice */
      if (!quiet) printf("%d %g\n",nd,T);
      if (nd==0) break;
      T*=1-2*T/N;
      //      if (exp(-2/T)<0.5/Sqr(N)) T*=1.5; /* .. try again */

      loopto (k,-4,4) exptab[k]=exp(k/T);

      loop (j,0,8*N) {
        i=irnd(N);

        memcpy(old,cfg[i].nbr,sizeof(cfg[i].nbr));
        Uold=ndef(i);
        a=irnd(4),b=irnd(4);
        k=cfg[i].nbr[a], cfg[i].nbr[a]=cfg[i].nbr[b],cfg[i].nbr[b]=k;
        Unew=ndef(i);
        if (rnd()>exptab[Uold-Unew])
          /* rejected: return back the old cfg */
          memcpy(cfg[i].nbr,old,sizeof(cfg[i].nbr)); } }

    printf("final dipole moment of the box:\n");
  } while (fabs(dipmq(ice->OO)-Mmid)>Merr);
}

void writecfg(char *fn)
{
  FILE *plb=fopen(string("%s.plb",fn),"wb");
  FILE *mol=fopen(string("%s.mol",fn),"wt");
  FILE *gol=fopen(string("%s.gol",fn),"wt");
  FILE *def=fopen(string("%s.def",fn),"wt");
  FILE *fix=fopen(string("%s.fix",fn),"wt");
  vector hdr,rn,rn2;
  double qOH=0.96/ice->OO;
  double qOM=0.1559/ice->OO;
  double qOL=0.8/ice->OO;
  double cutoff;
  int i;
  int nsites=strlen(SITES),isM=!!strchr(SITES,'M'),isL=!!strchr(SITES,'L');

  hdr[0]=N*nsites;
  hdr[1]=-3;
  fwrite(hdr,4,2,plb);
  fwrite(L  ,4,3,plb);
  cutoff=fmin(L[0],L[1]);
  cutoff=fmin(cutoff,L[2])/2;
  fprintf(def,"! %s\n\
! Mcell = %.6f %.6f %.6f = %g D (for mu_H2O=2.3052 D)\n\
n=%d N[0]=n\n\
L[0]=%.7f\n\
L[1]=%.7f\n\
L[2]=%.7f\n\
cutoff=%.7f LJcutoff=cutoff\n",
          args,
          VARG(M),sqrt(SQR(M)),
          N,VARG(L),cutoff);
  fclose(def);

  fprintf(mol,"ice %s\n\
\n\
parameter_set = dumb\n\
number_of_atoms = %d\n\
\n\
! Mcell = %.6f %.6f %.6f = %g D (for mu_H2O=2.3052 D)\n\
\n\
atoms\n\
! i   atom-id  a-type charge chir nbonds bound_atoms\n",
          args,N*strlen(SITES),VARG(M),sqrt(SQR(M)));

  fprintf(gol,"!sphere#1\n\
! %s\n\
%d\n",SITES,N*strlen(SITES));

  loop (i,0,N) {
    int hb=0,hl=0,l,n;
    char *c,*cc;

    for (c=SITES; *c; c++) {
      n=i*nsites+(int)(c-SITES);
      switch (*c) {
        case 'O':
          VV(hdr,=cfg[i].r)
          fprintf(mol,"%3d %3d-O O 0.0 0 %d",n,n,2+isM+2*isL);
          for (cc=SITES; *cc; cc++) if (strchr("HML",*cc))
            fprintf(mol," %d",i*nsites+(int)(cc-SITES));
          fprintf(mol,"\n");
          fprintf(gol,"R 1.2\n");
          fprintf(fix,"%4d %6.3f %6.3f %6.3f\n",n,hdr[0],hdr[1],hdr[2]);
          break;
        case 'H':
          if (hb>=2) Error("wrong SITES (too many H)");
          VV(rn,=cfg[cfg[i].nbr[hb]].r)
          loop (l,0,3) {
            if (rn[l]-cfg[i].r[l]>L[l]/2) rn[l]-=L[l];
            if (rn[l]-cfg[i].r[l]<-L[l]/2) rn[l]+=L[l]; }
          VVV(hdr,=(1-qOH)*cfg[i].r,+qOH*rn)
          hb++;
          fprintf(mol,"%3d %3d-H H 0.0 0 1",n,n);
          for (cc=SITES; *cc; cc++) if (*cc=='O')
            fprintf(mol," %d",i*nsites+(int)(cc-SITES));
          fprintf(mol,"\n");
          fprintf(gol,"W 0.7\n");
          break;
        case 'L':
          if (hl>=2) Error("wrong SITES (too many L)");
          VV(rn,=cfg[cfg[i].nbr[2+hl]].r)
          loop (l,0,3) {
            if (rn[l]-cfg[i].r[l]>L[l]/2) rn[l]-=L[l];
            if (rn[l]-cfg[i].r[l]<-L[l]/2) rn[l]+=L[l]; }
          VVV(hdr,=(1-qOL)*cfg[i].r,+qOL*rn)
          hl++;
          fprintf(mol,"%3d %3d-L L 0.0 0 1",n,n);
          for (cc=SITES; *cc; cc++) if (*cc=='O')
            fprintf(mol," %d",i*nsites+(int)(cc-SITES));
          fprintf(mol,"\n");
          fprintf(gol,"B 0.6\n");
          break;
        case 'M':
          VV(rn,=cfg[cfg[i].nbr[2]].r)
          loop (l,0,3) {
            if (rn[l]-cfg[i].r[l]>L[l]/2) rn[l]-=L[l];
            if (rn[l]-cfg[i].r[l]<-L[l]/2) rn[l]+=L[l]; }
          VVV(hdr,=(1-qOH)*cfg[i].r,+qOH*rn)
          VV(rn2,=cfg[cfg[i].nbr[3]].r)
          loop (l,0,3) {
            if (rn2[l]-cfg[i].r[l]>L[l]/2) rn2[l]-=L[l];
            if (rn2[l]-cfg[i].r[l]<-L[l]/2) rn2[l]+=L[l]; }
          VVVV(hdr,=(1+2*qOM)*cfg[i].r,-qOM*rn,-qOM*rn2)
          fprintf(mol,"%3d %3d-M M 0.0 0 1",n,n);
          for (cc=SITES; *cc; cc++) if (*cc=='O')
            fprintf(mol," %d",i*nsites+(int)(cc-SITES));
          fprintf(mol,"\n");
          fprintf(gol,"M 1.1\n");
          break;
        default:
          Error("bad SITES: only OHML accepted"); }
      VO(hdr,+=1) /* linked-cell dislikes negative coords */
      fwrite(hdr,4,3,plb); } }
  fclose(plb);
  fclose(mol);
  fclose(gol);
  fclose(fix);

  printf("molcfg -%d %s-water %s\n",N,SITES,fn);
  printf("box = %.7f %.7f %.7f\n",VARG(L));

  {
    double epsr=95; /* 0 C: 96.5; 91.5; */
    double eps0=8.8541878e-12;
    double k=1.3806504e-23;
    double V=L[0]*L[1]*L[2]*1e-30;
    double Debye=1./2.99792458e+29;

    double M=sqrt((epsr-1)*(2*epsr+1)/(3*epsr)*3*eps0*273*k*V);
    printf("M[epsr=%g] = %g D\n",epsr,M/Debye);
  }
}

struct cfg_s *cfg0;
long long int ndis=0,maxndis=0,info;

int nbrsof(int n)
{
  int i;

  if (n>=N) {
    ndis++;
    if (ndis>=info) { printf("%lld FOUND\n",ndis); info+=1000+info/10; }
    if (ndis<maxndis) writecfg(string("all-%lld",ndis));
    return 1; /* proton cfg found! */
  }

  cfg[n]=cfg0[n];

  loop (i,0,6) {
    nextcfg(n,i);
    if (ndef_lt(n)==0) nbrsof(n+1); }

  return 0;
}


int main(int narg,char **arg)
{
  int n,nn,nx,ny,nz,ix,iy,iz,i,j,k;
  double aa,V;

  if (narg<2) {
    fprintf(stderr,"Generate ice structure with proton disorder. Call by:\n\
  ice TYPE [NX[COORD] NY[COORD] NZ[COORD] [[-]SITES [fromM:toM]]\n\
TYPE = ice type, one of {Ic,Ih}\n\
NX,NY,NZ = how many times to repeat the basic cell (missing=only info on ice)\n\
COORD={x,y,z} denotes transformation of coordinates; default=xyz\n\
SITES = generate plb-file with given structure and order of sites\n\
        SITES=string of {H,H,O,M,L}; M=TIP4P-like site, L=lone pair\n\
-SITES = quiet (no protocol on annealing)\n\
fromM:toM = range of box dipole moments accepted (in D, mu_H2O=mu_TIP4P/2005)\n\
Environment:\n\
  if ICENAME given, it is used as basename, otherwise it is created\n\
  if ICEALL, all cfgs are generated, of these upto ICEALL are written\n\
Algorithm:\n\
  simulated annealing of initial randomly oriented water molecules until all\n\
  Bjerrum defects annihilate\n\
Caveats:\n\
  * the output structure is ideally tetrahedral\n\
  * water is tetrahedral (angle HOH=109.47 deg), HO=0.96, OM=0.18, OL=0.80\n\
  * the algorithm is inefficient for large lattices\n\
Example (basic cell repeated 5x in x, but written as y):\n\
  ice Ih 5y 3z 3x -HOMH\n\
See also:\n\
  lattice\n\
");
    exit(0); }

  rndinit(0,0);
  stringinit(256,6);

  if (!strcmp(arg[1],"Ih")) ice=&Ih;
  else if (!strcmp(arg[1],"Ic")) ice=&Ic;
  else Error("unsupported ice");

  if (getenv("ICERHO")) ice->OO*=cubrt(917.18/atof(getenv("ICERHO")));

  printf("ice %s basic cell:\n",arg[1]);
  printf("  %d molecules\n",ice->n);
  printf("  a=(%.7f,%.7f,%.7f)*%.7g\n",VARG(ice->a),ice->OO);

  VO(ice->a,*=ice->OO)
  V=ice->a[0]*ice->a[1]*ice->a[2]/ice->n;

  printf("   =(%.7f,%.7f,%.7f) A\n",VARG(ice->a));
  printf("  volume/molecule=%g A^3  density=%g kg/m^3\n",V,29915.07113/V);

  if (narg<3) exit(0);

  nx=atoi(arg[2]); ix=indx(arg[2],0);
  ny=atoi(arg[3]); iy=indx(arg[3],1);
  nz=atoi(arg[4]); iz=indx(arg[4],2);

  if (ix+iy+iz!=3) Error("bad transformation; either omit modifiers xyz or use each just once");

  N=ice->n*nx*ny*nz;
  L[ix]=ice->a[0]*nx;
  L[iy]=ice->a[1]*ny;
  L[iz]=ice->a[2]*nz;

  printf("basic cell multiplied by %dx%dx%d = %g %g %g lattice\n\
%d vertices, transformation=%c%c%c\n",
         nx,ny,nz,         L[ix],L[iy],L[iz],       N,               ix+'x',iy+'x',iz+'x');

  if (narg<6) {
    fprintf(stderr,"(specify parameter SITES to calculate the ice structure)\n");
    exit(0); }

  quiet=arg[5][0]=='-';

  if (narg>6) {
    double x=atof(arg[6]),to=1e10;
    char *c=strchr(arg[6],':');
    if (!c) Error("fromM and toM must be separated by ':'");
    to=atof(c+1);
    if (to>9e9) Error("toM out of range (9e9)");
    Mmid=(x+to)/2;
    Merr=(to-x)/1.9999999999;
    if (Merr<=0) Error("bad or zero dipole moment range fromM:toM");  }

  allocarrayzero(cfg,N);

  nn=0;
  loop (i,0,nx)
    loop (j,0,ny)
      loop (k,0,nz)
        loop (n,0,ice->n) {
          cfg[nn].r[ix]=ice->a[0]*(i+ice->r[n][0]);
          cfg[nn].r[iy]=ice->a[1]*(j+ice->r[n][1]);
          cfg[nn].r[iz]=ice->a[2]*(k+ice->r[n][2]);
          nn++; }
  if (nn!=N) Error("nn!=N");

//  loop (i,0,N) loop (j,0,N) printf("%g\n",dist(cfg[i].r,cfg[j].r));

  aa=Sqr(ice->OO)*1.2; /* to cover all 1st neighbors but no 2nd neighbor */

  loop (i,0,N) loop (j,0,i)
    if (ddist(cfg[i].r,cfg[j].r)<aa) {
      if (cfg[i].nnbr<4) cfg[i].nbr[cfg[i].nnbr++]=j;
      else Error("more than 4 neighbors");
      if (cfg[j].nnbr<4) cfg[j].nbr[cfg[j].nnbr++]=i;
      else Error("more than 4 neighbors"); }

  loop (i,0,N) if (cfg[i].nnbr!=4) Error("no 4 neighbors (too small lattice?)");

  args=strdup(string("%s %s %s %s",arg[1],arg[2],arg[3],arg[4]));

  SITES=arg[5]+quiet;

  if (getenv("ICEALL")) {
    info=maxndis=atoi(getenv("ICEALL"));
    allocarray(cfg0,N);
    copyarray(cfg0,cfg,N);
    nbrsof(0);
    printf("%lld proton disordered configurations found\n",ndis);
  }
  else {
    annealing();
    writecfg(getenv("ICENAME")?getenv("ICENAME"):string("%s%d%d%d%s",arg[1],nx,ny,nz,SITES));
  }

  return 0;
}
