/*
  fortran-like formatted input

  synopsis:

    int forscanf(FILE *file,const char *format,...);

  format field    corresponding parameter
    
    %9i %9d       pointer to integer    
    %9li %9ld     pointer to integer    
    %9f           pointer to float
    %9lf          pointer do double
    %9a           array of char of length [9] (no \0 appended)
    %9s           array of char of length [9+1] (\0 appended, blanks removed)
    <space>       ignore space (fortran 1X)
    \n            ignore all data to end of input line

  the number after % is the length of the field irrespective of any
  blank characters (thus, spaces are included into strings!)

  max field length is 80

  tabulators are not and will not be supported (= they are treated as
  ordinary characters)

  number of scanned items is returned

  if \n is encountered before formats have been scanned, spaces are
  appended

  format %9a reads 9 characters 

  format %9s reads 9 characters, removes leading and trailing spaces
  (' ', not tabs etc.) and appends \0; spaces inside the string are left
  (e.g., " A B " -> "A B" (neded by \0) if read by "%5s"

*/

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int forscanf(FILE *file,const char *format,...)
{
va_list args;
char *fmt=(char*)format;
int c,j,i=0,len,eol=0,l;

char field[82],*f;

#define mygetc { \
  if (eol) c='\n'; else c=fgetc(file); \
  if (c<0) { i=-1; goto done; } \
  if (c=='\n') { eol=1; c=' '; } }


va_start(args,format);

again:

switch (*fmt) {

  case 0: goto done;

  case '\n':
    if (eol) { eol=0; break; /* already eol */ }
    do {
      c=fgetc(file);
      if (c<0) { i=-1; goto done; }
      } while (c!='\n');
    eol=0;
    break;

  case ' ':   
    mygetc
    break;

  case '%':
    len=atoi(++fmt);
    if (len<0 || len>80) { i=-3; goto done; }
    while (*fmt>='0' && *fmt<='9') fmt++;

    for (j=0; j<len; j++) {
      mygetc
      field[j]=c; }
    field[j]=0;

    l=0;
    if (*fmt=='l') { l=1; fmt++; } /* arg is long */

    switch (*fmt) {

      case 'd': case 'i':
        if (l) *va_arg(args,long*)=atol(field);
	else *va_arg(args,int*)=atoi(field);
	break;

      case 'f':
        if (l) *va_arg(args,double*)=atof(field);
	else *va_arg(args,float*)=atof(field);
	break;

      case 'a':
	memcpy(va_arg(args,char*),field,len);
	break;

      case 's':
	f=field+len-1;
	while (*f==' ' && f>=field) *f--=0;
	f=field;
	while (*f && *f==' ') f++;
	strcpy(va_arg(args,char*),f);
	break;

      default:
	i=-4; goto done;
      }
    i++;
    break;

  default:
    i=-5; goto done;
	
  }

fmt++;
goto again;

done:
va_end(args);

return i;

}
