#define COR_BONDS 0 /* correct bonds (Lagrange), also SHAKE */
#define COR_VELOC 1 /* correct velocity constraints (Lagrange) */
#define LAGR_MULT 2 /* Lagrange multipliers */
#define COR_BOTH  3 /* Lagrange: -c2, both (?) */
#define COR_LAST  4 /* for SHAKE autoset of omegac */
#define NCONSTRIT 5 /* size */

typedef struct {
  int nit[NCONSTRIT]; /* numbers of iterations, indices see above */
  double omegac;      /* active omegac (SHAKE) */
  double omega;       /* active omegac/2 (SHAKE) */
  double omegaclast;  /* last (refers to nit[COR_LAST]) */
  double d;           /* step for omegac optimization */
  double weight;      /* weight for omegac optimization */
  } constrit_t;
extern constrit_t *constrit; /* [nspec] */  
void initconstrit(double omegac);
void doconstrit(double omegac);

extern double vconstrainterror;
double constrainterror(ToIntPtr A,ToIntPtr V);
int Scorrect(molecule_t *m, vector *r, vector *rpvel, double eps);
void Lcorrect(ToIntPtr B, ToIntPtr V);
int normalize(int m);
void sortmolecules(int sort);
void bounddr(double drmax);
void setdrift(void);

char *prtxyz(int xyz);

void removedrifts(int pr);
#ifdef SLAB
void centerslabanddrift(int pr);
#endif

double rescalecfg(ToIntPtr A,int mode, double qx,double qy,double qz);
int rhorescale(int rescale,double tau,vector finalL,double halftcyc);
double scaling(double tau,int noint,double factor,double maxscale);

void addEnnit(double *En_nit,int n,double add);
#ifdef DISTURB
void Disturb(int ia,double ran);
#endif
void distancecheck(void);
int debugcfg(int quit);
void zeroEn(void);

void depend_r(ToIntPtr A);
void depend_f(ToIntPtr A,ToIntPtr B);

int iscube(void);

int centergroups(ToIntPtr A,int valence);
void addshift(vector shift);

#define RESCALE_X        1
#define RESCALE_Y        2
#define RESCALE_Z        4
#define RESCALE_XYZ      7
#define RESCALE_CM       8 /* center-of-mass based rescaling */
#define RESCALE_PT      16 /* use PRESSURETENSOR (components) */
#define RESCALE_XisY    32 /* modifier of RESCALE_PT (xscale=yscale) */
#define RESCALE_XisYisZ 64 /* uniform (xscale=yscale=zscale) */
#define RESCALE_L      256 /* rescale also box */
#define RESCALE_WALL   512 /* WALL version only */
#define RESCALE_SLAB  1024 /* SLAB version only (virtual area change) */

#define RESCALE_CLEAVE  2048 /* z-box barostat w. cleaving boundaries */
#if defined(SLAB) && SLAB & 2
void cleavescaling(double *Pscale,double tau,int noint,double factor,double maxscale);
#endif
